@@ -10,6 +10,7 @@
 ##' we can find in the fitted model are presented.  Otherwise, a subset
 ##' of parameter sets can be chosen by the user.
 ##' \itemize{
+##' \item "composites" are predictor coefficients in formative constructs
 ##' \item "loadings" are the factor loadings in the model.
 ##' \item "slopes" are the regression slopes in the model.
 ##' \item "intercepts" are the estimated constants in the measurement
@@ -39,19 +40,20 @@
 ##'     side.
 ##' @param paramSets Parameter sets to be included for each fitted
 ##'     object.  Valid values of the vector are \code{"all"} or a any
-##'     of the following: \code{c("loadings", "slopes", "intercepts",
-##'     "residualvariances", "residualcovariances", "latentmeans",
-##'     "latentvariances", "latentcovariances", "thresholds",
-##'     "constructed", "fits")}. Default is "all", any of the
-##'     estimates present in the fitted model that are listed in the
-##'     previous sentence will be included in the output. For the sake
-##'     of simplicity, we now allow one vector here, which applies to
-##'     all models in the object list.
+##'     of the following: \code{c("composites", "loadings", "slopes",
+##'     "intercepts", "residualvariances", "residualcovariances",
+##'     "latentmeans", "latentvariances", "latentcovariances",
+##'     "thresholds", "constructed", "fits")}. Default is "all", any
+##'     of the estimates present in the fitted model that are listed
+##'     in the previous sentence will be included in the output. For
+##'     the sake of simplicity, we now allow one vector here, which
+##'     applies to all models in the object list.
 ##' @param paramSetLabels Named vector, used to supply alternative
 ##'     pretty printing labels for parameter sets. The default values
-##'     are \code{c("loadings"= "Factor Loadings", "slopes" =
-##'     "Regression Slopes", "intercepts" = "Intercepts", "means"=
-##'     "Means", "residualvariances" = "Residual Variances",
+##'     are \code{c("composites" = "Composites",
+##'     "loadings"= "Factor Loadings", "slopes" = "Regression Slopes",
+##'     "intercepts" = "Intercepts", "means"= "Means",
+##'     "residualvariances" = "Residual Variances",
 ##'     "residualcovariances" = "Residual Covariances", "variances" =
 ##'     "Variances", "latentvariances" = "Latent Variances",
 ##'     "latentcovariances" = "Latent Covariances", "latentmeans" =
@@ -246,7 +248,7 @@ semTable <- function(object, file = NULL, paramSets = "all", paramSetLabels,
 
     ## relabel the "paramType" column in a parTable data frame
     ##
-    ## Re-lable the "paramType", the parameterSet indicator for each
+    ## Re-label the "paramType", the parameterSet indicator for each
     ## row in a parTable. Identifies "residualvariances",
     ## "residualcovariances", "latentvariances" "latentcovariances",
     ## "intercepts" and "latentmeans"
@@ -255,7 +257,8 @@ semTable <- function(object, file = NULL, paramSets = "all", paramSetLabels,
     ## @return 
     ## @author Paul Johnson
     insertParamTypes <- function(parameters){
-        paramops <- c("=~" = "loadings", "~" = "slopes", "~1" = "means",
+        paramops <- c("=~" = "loadings", "<~" = "composites",
+                      "~" = "slopes", "~1" = "means",
                       "~~" = "variances", "|" = "thresholds",
                       ":=" = "constructed", "==" = "constraints")
 
@@ -399,7 +402,7 @@ semTable <- function(object, file = NULL, paramSets = "all", paramSetLabels,
         blocktitle
     }
 
-    ##works for paramSets = "loadings" or "slopes"
+    ##works for paramSets = "composites", "loadings" or "slopes"
     loadingMaker <- function(parameters, paramType = "loadings", colLabels, modelName) {
         report <- names(colLabels[[modelName]])
         totalNcolumns <- min(9,  length(unname(unlist(colLabels))))
@@ -492,7 +495,12 @@ semTable <- function(object, file = NULL, paramSets = "all", paramSetLabels,
         ## "Scaled chisq"
         nOfTests <- length(object@Fit@test)
         chimeas <- object@Fit@test[[nOfTests]]
-        if (nOfTests > 1) newdf[1 , "col1"] <- "Scaled _CHI2_"
+        ## address problem that similar named rows "chisq" were confused.
+        if (nOfTests > 1){
+            rname <- "chisqscaled"
+            newdf[1 , "col1"] <- "Scaled _CHI2_"
+            rownames(newdf)[1] <- rname
+        }
         chimeas$stat <- frnd(chimeas$stat)
         chimeas$pvalue <- frnd(chimeas$pvalue, 3,  3)
         chimeas$pvalue <- gsub("0\\.", "\\.", chimeas$pvalue)
@@ -556,12 +564,14 @@ semTable <- function(object, file = NULL, paramSets = "all", paramSetLabels,
         ## Still must treat loadings and slopes differently
         reslt <- list()
         for(jj in names(paramTableSplit)){
-            if (jj %in% c("loadings", "slopes")){
+            if (jj %in% c("composites", "loadings", "slopes")){
                 
                 info <- loadingMaker(paramTableSplit[[jj]], paramType = jj,
                                      colLabels, modelName)
                 if (!is.null(info)){
-                    attr(info, "blocktitle") <- makeSubtableTitle(paramSetLabels["loadings"],
+                    ## 20190517: following looks like error, why fixed at "loadings"
+                    ## attr(info, "blocktitle") <- makeSubtableTitle(paramSetLabels["loadings"],
+                    attr(info, "blocktitle") <- makeSubtableTitle(paramSetLabels[jj],
                                                                   colnum = 2,
                                                                   width = length(report))
                     class(info) <- c("trowsList", class(info))
@@ -597,7 +607,8 @@ semTable <- function(object, file = NULL, paramSets = "all", paramSetLabels,
         ## chisq is a special presentation, comes back as a one row data frame
         if ("chisq" %in% names(fitmeas)){
             yy <- getChiSq(object, report)["chisq", ]
-            info["chisq", ] <- yy
+            rname <- rownames(yy)[1]
+            info[rname, ] <- yy
         }
         
         attr(info, "blocktitle") <- makeSubtableTitle(paramSetLabels["fits"],
@@ -664,7 +675,7 @@ semTable <- function(object, file = NULL, paramSets = "all", paramSetLabels,
             tablList <- lapply(paramList, function(x) x[[jj]])
             ## treat loadings and slopes differently because they are
             ## lists of tables
-            if(jj %in% c("loadings", "slopes")){
+            if(jj %in% c("composites", "loadings", "slopes")){
                 ## get the varnames
                 varnames <- unique(unlist(lapply(tablList, names)))
                 hh <- list()
@@ -765,6 +776,7 @@ semTable <- function(object, file = NULL, paramSets = "all", paramSetLabels,
     
     ## paramSetLabels default.
     paramx <- c("loadings" = "Factor Loadings",
+                "composites" = "Composites", 
                 "slopes" = "Regression Slopes",
                 "intercepts" = "Intercepts",
                 "means" = "Means",
